from MorphologicalAnalysis.State cimport State


cdef class FiniteStateMachine:

    cdef list __states
    cdef dict __transitions

    cpdef bint isValidTransition(self, str transition)
    cpdef list getStates(self)
    cpdef State getState(self, str name)
    cpdef addTransition(self, State fromState, State toState, str _with, str withName, toPos=*)
    cpdef list getTransitions(self, State state)


from Dictionary.Word cimport Word
from MorphologicalAnalysis.FsmParse cimport FsmParse

cdef class FsmParseList:

    cdef list __fsmParses

    cpdef int size(self)
    cpdef addFsmParse(self, FsmParse fsmParse)
    cpdef FsmParse getFsmParse(self, int index)
    cpdef str rootWords(self)
    cpdef reduceToParsesWithSameRootAndPos(self, Word currentWithPos)
    cpdef FsmParse getParseWithLongestRootWord(self)
    cpdef reduceToParsesWithSameRoot(self, str currentRoot)
    cpdef list constructParseListForDifferentRootWithPos(self)
    cpdef str parsesWithoutPrefixAndSuffix(self)



from Dictionary.TxtWord cimport TxtWord
from MorphologicalAnalysis.FsmParse cimport FsmParse
from MorphologicalAnalysis.State cimport State


cdef class Transition:

    cdef State __toState
    cdef str __with, __withName, __formationToCheck, __toPos

    cpdef State toState(self)
    cpdef str toPos(self)
    cpdef bint transitionPossibleForString(self, str currentSurfaceForm, str realSurfaceForm)
    cpdef bint transitionPossibleForParse(self, FsmParse currentFsmParse)
    cpdef bint transitionPossibleForWord(self, TxtWord root, State fromState)
    cpdef str __beforeLastVowel(self, str stem)
    cpdef str __lastVowel(self, str stem)
    cpdef str __lastPhoneme(self, str stem)
    cpdef str __withFirstChar(self)
    cpdef bint __startWithVowelorConsonantDrops(self)
    cpdef bint softenDuringSuffixation(self, TxtWord root)
    cpdef str makeTransitionNoStartState(self, TxtWord root, str stem)
    cpdef str makeTransition(self, TxtWord root, str stem, State startState)
    cpdef str __resolveD(self, TxtWord root, str formation)
    cpdef __resolveA(self, TxtWord root, str formation, bint rootWord)
    cpdef __resolveH(self, TxtWord root, str formation, bint beginningOfSuffix, bint specialCaseTenseSuffix,
                   bint rootWord)
    cpdef str __resolveC(self, str formation)
    cpdef str __resolveS(self, str formation)
    cpdef str __resolveSh(self, str formation)
    cpdef str withName(self)



from Corpus.Sentence cimport Sentence
from Dictionary.Trie.Trie cimport Trie
from Dictionary.TxtDictionary cimport TxtDictionary
from DataStructure.Cache.LRUCache cimport LRUCache
from Dictionary.TxtWord cimport TxtWord

from MorphologicalAnalysis.FsmParse cimport FsmParse
from MorphologicalAnalysis.FiniteStateMachine cimport FiniteStateMachine
from MorphologicalAnalysis.MetamorphicParse cimport MetamorphicParse
from MorphologicalAnalysis.MorphologicalParse cimport MorphologicalParse


cdef class FsmMorphologicalAnalyzer:

    cdef Trie __dictionaryTrie
    cdef FiniteStateMachine __finiteStateMachine
    cdef TxtDictionary __dictionary
    cdef LRUCache __cache
    cdef dict __mostUsedPatterns
    cdef dict __parsedSurfaceForms

    cpdef addParsedSurfaceForms(self, str fileName)
    cpdef set getPossibleWords(self, MorphologicalParse morphologicalParse, MetamorphicParse metamorphicParse)
    cpdef TxtDictionary getDictionary(self)
    cpdef FiniteStateMachine getFiniteStateMachine(self)
    cpdef bint __isPossibleSubstring(self, str shortString, str longString, TxtWord root)
    cpdef __initializeParseList(self, list fsmParse, TxtWord root, bint isProper)
    cpdef __initializeParseListFromRoot(self, list parseList, TxtWord root, bint isProper)
    cpdef list __initializeParseListFromSurfaceForm(self, str surfaceForm, bint isProper)
    cpdef __addNewParsesFromCurrentParse(self, FsmParse currentFsmParse, list fsmParse, maxLengthOrSurfaceForm,
                                       TxtWord root)
    cpdef bint __parseExists(self, list fsmParse, str surfaceForm)
    cpdef list __parseWord(self, list fsmParse, maxLengthOrSurfaceForm)
    cpdef list morphologicalAnalysisRoot(self, str surfaceForm, TxtWord root, state=*)
    cpdef list generateAllParses(self, TxtWord root, int maxLength)
    cpdef str replaceRootWord(self, FsmParse parse, TxtWord newRoot)
    cpdef Sentence replaceWord(self, Sentence original, str previousWord, str newWord)
    cpdef bint __analysisExists(self, TxtWord rootWord, str surfaceForm, bint isProper)
    cpdef list __analysis(self, str surfaceForm, bint isProper)
    cpdef bint patternMatches(self, str expr, str value)
    cpdef bint isProperNoun(self, str surfaceForm)
    cpdef morphologicalAnalysis(self, sentenceOrSurfaceForm)
    cpdef robustMorphologicalAnalysis(self, sentenceOrSurfaceForm)
    cpdef bint __isInteger(self, str surfaceForm)
    cpdef bint __isDouble(self, str surfaceForm)
    cpdef bint __isNumber(self, str surfaceForm)
    cpdef bint __isPercent(self, str surfaceForm)
    cpdef bint __isTime(self, str surfaceForm)
    cpdef bint __isRange(self, str surfaceForm)
    cpdef bint __isDate(self, str surfaceForm)
    cpdef str __toLower(self, str surfaceForm)
    cpdef bint morphologicalAnalysisExists(self, TxtWord rootWord, str surfaceForm)



from Dictionary.Word cimport Word

from MorphologicalAnalysis.InflectionalGroup cimport InflectionalGroup
from MorphologicalAnalysis.MorphologicalTag import MorphologicalTag


cdef class MorphologicalParse:

    cdef list inflectionalGroups
    cdef Word root

    cpdef Word getWord(self)
    cpdef str getTransitionList(self)
    cpdef str getInflectionalGroupString(self, int index)
    cpdef InflectionalGroup getInflectionalGroup(self, int index)
    cpdef InflectionalGroup getLastInflectionalGroup(self)
    cpdef str getTag(self, int index)
    cpdef int tagSize(self)
    cpdef int size(self)
    cpdef InflectionalGroup firstInflectionalGroup(self)
    cpdef InflectionalGroup lastInflectionalGroup(self)
    cpdef Word getWordWithPos(self)
    cpdef str getPos(self)
    cpdef str getRootPos(self)
    cpdef str lastIGContainsCase(self)
    cpdef bint lastIGContainsTag(self, object tag)
    cpdef bint lastIGContainsPossessive(self)
    cpdef bint isCapitalWord(self)
    cpdef bint isNoun(self)
    cpdef bint isVerb(self)
    cpdef bint isRootVerb(self)
    cpdef bint isAdjective(self)
    cpdef bint isProperNoun(self)
    cpdef bint isPunctuation(self)
    cpdef bint isCardinal(self)
    cpdef bint isOrdinal(self)
    cpdef bint isReal(self)
    cpdef bint isNumber(self)
    cpdef bint isTime(self)
    cpdef bint isDate(self)
    cpdef bint isHashTag(self)
    cpdef bint isEmail(self)
    cpdef bint isPercent(self)
    cpdef bint isFraction(self)
    cpdef bint isRange(self)
    cpdef bint isPlural(self)
    cpdef bint isAuxiliary(self)
    cpdef bint containsTag(self, object tag)
    cpdef str getTreePos(self)
    cpdef str getPronType(self)
    cpdef str getNumType(self)
    cpdef str getReflex(self)
    cpdef str getNumber(self)
    cpdef str getCase(self)
    cpdef str getDefinite(self)
    cpdef str getDegree(self)
    cpdef str getPolarity(self)
    cpdef str getPerson(self)
    cpdef str getVoice(self)
    cpdef str getAspect(self)
    cpdef str getTense(self)
    cpdef str getMood(self)
    cpdef str getVerbForm(self)
    cpdef list getUniversalDependencyFeatures(self, str uPos)
    cpdef str getUniversalDependencyPos(self)



from distutils.core import setup
from Cython.Build import cythonize

setup(
    ext_modules=cythonize(["MorphologicalAnalysis/*.pyx", "DisambiguationCorpus/*.pyx"],
                          compiler_directives={'language_level': "3"}),
    name='NlpToolkit-MorphologicalAnalysis-Cy',
    version='1.0.16',
    packages=['MorphologicalAnalysis', 'DisambiguationCorpus'],
    package_data={'MorphologicalAnalysis': ['*.pxd', '*.pyx', '*.c', '*.py'],
                  'DisambiguationCorpus': ['*.pxd', '*.pyx', '*.c', '*.py']},
    url='https://github.com/StarlangSoftware/TurkishMorphologicalAnalysis-Cy',
    license='',
    author='olcaytaner',
    author_email='olcay.yildiz@ozyegin.edu.tr',
    description='Turkish Morphological Analysis',
    install_requires=['NlpToolkit-Dictionary-Cy', 'NlpToolkit-Corpus-Cy', 'NlpToolkit-DataStructure-Cy']
)
